@@ -5,6 +5,7 @@ from discord.ext import commands
 from . import http
 from . import model
 from .utils import manage_commands
+from inspect import iscoroutinefunction
 
 
 class SlashCommand:
@@ -23,6 +24,7 @@ class SlashCommand:
     :ivar req: :class:`.http.SlashCommandRequest` of this client.
     :ivar logger: Logger of this client.
     :ivar auto_register: Whether to register commands automatically.
+    :ivar has_listener: Whether discord client has listener add function.
     """
     def __init__(self,
                  client: typing.Union[discord.Client, commands.Bot],
@@ -31,50 +33,91 @@ class SlashCommand:
         self._discord = client
         self.commands = {}
         self.subcommands = {}
-        self.req = http.SlashCommandRequest()
         self.logger = logging.getLogger("discord_slash")
+        self.req = http.SlashCommandRequest(self.logger)
         self.auto_register = auto_register
         if self.auto_register:
-            self.logger.warning("auto_register is NOT implemented! Please manually add commands to Discord API.")
-        if not isinstance(client, commands.Bot) and not override_type:
+            self._discord.loop.create_task(self.register_all_commands())
+        if not isinstance(client, commands.Bot) and not isinstance(client, commands.AutoShardedBot) and not override_type:
             self.logger.info("Detected discord.Client! Overriding on_socket_response.")
             self._discord.on_socket_response = self.on_socket_response
+            self.has_listener = False
         else:
             self._discord.add_listener(self.on_socket_response)
+            self.has_listener = True
 
     def remove(self):
         self._discord.remove_listener(self.on_socket_response)
 
+    async def register_all_commands(self):
+        await self._discord.wait_until_ready() # In case commands are still not registered to SlashCommand.
+        self.logger.info("Registering commands...")
+        for x in self.commands.keys():
+            selected = self.commands[x]
+            if selected["has_subcommands"] and "func" not in selected.keys():
+                # Just in case it has subcommands but also has base command.
+                # More specific, it will skip if it has subcommands and doesn't have base command coroutine.
+                self.logger.debug("Skipping registering subcommands.")
+                continue
+            if selected["guild_ids"]:
+                for y in selected["guild_ids"]:
+                    await manage_commands.add_slash_command(self._discord.user.id,
+                                                            self._discord.http.token,
+                                                            y,
+                                                            x,
+                                                            selected["description"],
+                                                            selected["api_options"])
+            else:
+                await manage_commands.add_slash_command(self._discord.user.id,
+                                                        self._discord.http.token,
+                                                        None,
+                                                        x,
+                                                        selected["description"],
+                                                        selected["api_options"])
+        self.logger.info("Completed registering all commands!")
+
     def add_slash_command(self,
                           cmd,
                           name: str = None,
                           description: str = None,
                           auto_convert: dict = None,
-                          guild_ids: list = None,
+                          guild_ids: typing.List[int] = None,
                           options: list = None,
                           has_subcommands: bool = False):
         """
         Registers slash command to SlashCommand.
 
         :param cmd: Command Coroutine.
+        :type cmd: Coroutine
         :param name: Name of the slash command. Default name of the coroutine.
+        :type name: str
         :param description: Description of the slash command. Default ``None``.
+        :type description: str
         :param auto_convert: Dictionary of how to convert option values. Default ``None``.
+        :type auto_convert: dict
         :param guild_ids: List of Guild ID of where the command will be used. Default ``None``, which will be global command.
-        :param options: Options of the slash command. This will affect ``auto_convert`` and command data at Discord API. Default ``None``.
+        :type guild_ids: List[int]
+        :param options: Options of the slash command.
+        :type options: list
         :param has_subcommands: Whether it has subcommand. Default ``False``.
-        :return: ``None``
+        :type has_subcommands: bool
         """
+        name = cmd.__name__ if not name else name
+        name = name.lower()
+        if name in self.commands.keys():
+            tgt = self.subcommands[name]
+            has_subcommands = tgt["has_subcommands"]
+            guild_ids += tgt["guild_ids"]
         _cmd = {
             "func": cmd,
-            "description": description,
+            "description": description if description else "No description.",
             "auto_convert": auto_convert,
             "guild_ids": guild_ids,
-            "api_options": options,
+            "api_options": options if options else [],
             "has_subcommands": has_subcommands
         }
-        self.commands[cmd.__name__ if not name else name] = _cmd
-        self.logger.debug(f"Added command `{cmd.__name__ if not name else name}`")
+        self.commands[name] = _cmd
+        self.logger.debug(f"Added command `{name}`")
 
     def add_subcommand(self,
                        cmd,
@@ -83,20 +126,29 @@ class SlashCommand:
                        name=None,
                        description: str = None,
                        auto_convert: dict = None,
-                       guild_ids: list = None):
+                       guild_ids: typing.List[int] = None):
         """
         Registers subcommand to SlashCommand.
 
-        :param cmd:
-        :param base:
-        :param subcommand_group:
-        :param name:
-        :param description:
-        :param auto_convert:
-        :param guild_ids:
-        :return:
+        :param cmd: Subcommand Coroutine.
+        :type cmd: Coroutine
+        :param base: Name of the base command.
+        :type base: str
+        :param subcommand_group: Name of the subcommand group, if any. Default ``None`` which represents there is no sub group.
+        :type subcommand_group: str
+        :param name: Name of the subcommand. Default name of the coroutine.
+        :type name: str
+        :param description: Description of the subcommand. Default ``None``.
+        :type description: str
+        :param auto_convert: Dictionary of how to convert option values. Default ``None``.
+        :type auto_convert: dict
+        :param guild_ids: List of guild ID of where the command will be used. Default ``None``, which will be global command.
+        :type guild_ids: List[int]
         """
+        base = base.lower()
+        subcommand_group = subcommand_group.lower() if subcommand_group else subcommand_group
         name = cmd.__name__ if not name else name
+        name = name.lower()
         _cmd = {
             "guild_ids": guild_ids,
             "has_subcommands": True
@@ -117,6 +169,7 @@ class SlashCommand:
             self.subcommands[base][subcommand_group][name] = _sub
         else:
             self.subcommands[base][name] = _sub
+        self.logger.debug(f"Added subcommand `{base} {subcommand_group if subcommand_group else ''} {cmd.__name__ if not name else name}`")
 
     def slash(self,
               *,
@@ -124,8 +177,8 @@ class SlashCommand:
               description: str = None,
               auto_convert: dict = None,
               guild_id: int = None,
-              guild_ids: list = None,
-              options: list = None):
+              guild_ids: typing.List[int] = None,
+              options: typing.List[dict] = None):
         """
         Decorator that registers coroutine as a slash command.\n
         All decorator args must be passed as keyword-only args.\n
@@ -162,11 +215,17 @@ class SlashCommand:
              "option_channel": "CHANNEL"} # and all upper case.
 
         :param name: Name of the slash command. Default name of the coroutine.
+        :type name: str
         :param description: Description of the slash command. Default ``None``.
+        :type description: str
         :param auto_convert: Dictionary of how to convert option values. Default ``None``.
+        :type auto_convert: dict
         :param guild_id: Deprecated. Use ``guild_ids`` instead.
-        :param guild_ids: Guild ID of where the command will be used. Default ``None``, which will be global command.
+        :type guild_id: int
+        :param guild_ids: List of Guild ID of where the command will be used. Default ``None``, which will be global command.
+        :type guild_ids: List[int]
         :param options: Options of the slash command. This will affect ``auto_convert`` and command data at Discord API. Default ``None``.
+        :type options: List[dict]
         """
 
         if guild_id:
@@ -178,7 +237,7 @@ class SlashCommand:
             auto_convert = {}
             for x in options:
                 if x["type"] < 3:
-                    raise Exception("Currently subcommand is NOT supported.")
+                    raise Exception("Please use `subcommand()` decorator for subcommands!")
                 auto_convert[x["name"]] = x["type"]
 
         def wrapper(cmd):
@@ -193,37 +252,52 @@ class SlashCommand:
                    name=None,
                    description: str = None,
                    auto_convert: dict = None,
-                   guild_ids: int = None):
+                   guild_ids: typing.List[int] = None):
         """
         Decorator that registers subcommand.\n
-        Unlike discord.py, you don't need base command.\n
-        Not implemented.
+        Unlike discord.py, you don't need base command.
 
         Example:
 
         .. code-block:: python
+
+            # /group say <str>
             @slash.subcommand(base="group", name="say")
             async def _group_say(ctx, _str):
                 await ctx.send(content=_str)
 
+            # /group kick user <user>
+            @slash.subcommand(base="group",
+                              subcommand_group="kick",
+                              name="user",
+                              auto_convert={"user": "user"})
+            async def _group_kick_user(ctx, user):
+                ...
+
         .. note::
             Unlike normal slash command, this doesn't support ``options`` arg, since it will be very complicated.\n
             Also, subcommands won't be automatically registered to Discord API even if you set ``auto_register`` to ``True``.
 
         :param base: Name of the base command.
+        :type base: str
         :param subcommand_group: Name of the subcommand group, if any. Default ``None`` which represents there is no sub group.
+        :type subcommand_group: str
         :param name: Name of the subcommand. Default name of the coroutine.
+        :type name: str
         :param description: Description of the subcommand. Default ``None``.
+        :type description: str
         :param auto_convert: Dictionary of how to convert option values. Default ``None``.
+        :type auto_convert: dict
         :param guild_ids: List of guild ID of where the command will be used. Default ``None``, which will be global command.
-        :return:
+        :type guild_ids: List[int]
         """
-        raise NotImplementedError
+
         def wrapper(cmd):
+            self.add_subcommand(cmd, base, subcommand_group, name, description, auto_convert, guild_ids)
             return cmd
         return wrapper
 
-    def process_options(self, guild: discord.Guild, options: list, auto_convert: dict) -> list:
+    async def process_options(self, guild: discord.Guild, options: list, auto_convert: dict) -> list:
         """
         Processes Role, User, and Channel option types to discord.py's models.
 
@@ -240,7 +314,11 @@ class SlashCommand:
             return [x["value"] for x in options]
         if not auto_convert:
             return [x["value"] for x in options]
-        converters = [guild.get_member, guild.get_role, guild.get_role]
+        converters = [
+            [guild.get_member, guild.fetch_member],
+            guild.get_channel,
+            guild.get_role]
+
         types = {
             "user": 0,
             "USER": 0,
@@ -261,8 +339,23 @@ class SlashCommand:
         for x in options:
             selected = x
             if selected["name"] in auto_convert.keys():
+                if auto_convert[selected["name"]] not in types.keys():
+                    to_return.append(selected["value"])
+                    continue
                 loaded_converter = converters[types[auto_convert[selected["name"]]]]
-                to_return.append(loaded_converter(int(selected["value"])))
+                if isinstance(loaded_converter, list):
+                    cache_first = loaded_converter[0](int(selected["value"]))
+                    if cache_first:
+                        to_return.append(cache_first)
+                        continue
+                    loaded_converter = loaded_converter[1]
+                try:
+                    to_return.append(await loaded_converter(int(selected["value"]))) \
+                        if iscoroutinefunction(loaded_converter) else \
+                        to_return.append(loaded_converter(int(selected["value"])))
+                except (discord.Forbidden, discord.HTTPException):
+                    self.logger.warning("Failed fetching user! Passing ID instead.")
+                    to_return.append(int(selected["value"]))
         return to_return
 
     async def on_socket_response(self, msg):
@@ -279,23 +372,95 @@ class SlashCommand:
         to_use = msg["d"]
         if to_use["data"]["name"] in self.commands.keys():
             selected_cmd = self.commands[to_use["data"]["name"]]
-            ctx = model.SlashContext(self.req, to_use, self._discord)
+            ctx = model.SlashContext(self.req, to_use, self._discord, self.logger)
             if selected_cmd["guild_ids"]:
                 if ctx.guild.id not in selected_cmd["guild_ids"]:
                     return
             if selected_cmd["has_subcommands"]:
                 return await self.handle_subcommand(ctx, to_use)
-            args = self.process_options(ctx.guild, to_use["data"]["options"], selected_cmd["auto_convert"]) \
+            args = await self.process_options(ctx.guild, to_use["data"]["options"], selected_cmd["auto_convert"]) \
                 if "options" in to_use["data"] else []
-            self.logger.debug(f"Command {to_use['data']['name']} invoked.")
-            await selected_cmd["func"](ctx, *args)
+            self._discord.dispatch("slash_command", ctx)
+            try:
+                await selected_cmd["func"](ctx, *args)
+            except Exception as ex:
+                await self.on_slash_command_error(ctx, ex)
 
     async def handle_subcommand(self, ctx: model.SlashContext, data: dict):
         """
         Coroutine for handling subcommand.
-        Not implemented.
 
-        :param ctx:
-        :param data:
+        .. warning::
+            Do not manually call this.
+
+        :param ctx: :class:`.model.SlashContext` instance.
+        :param data: Gateway message.
         """
-        pass
+        if data["data"]["name"] not in self.subcommands.keys():
+            return
+        base = self.subcommands[data["data"]["name"]]
+        sub = data["data"]["options"][0]
+        sub_name = sub["name"]
+        if sub_name not in base.keys():
+            return
+        ctx.subcommand = sub_name
+        sub_opts = sub["options"] if "options" in sub else []
+        for x in sub_opts:
+            if "options" in x.keys() or "value" not in x.keys():
+                sub_group = x["name"]
+                if sub_group not in base[sub_name].keys():
+                    return
+                ctx.subcommand_group = sub_group
+                selected = base[sub_name][sub_group]
+                args = await self.process_options(ctx.guild, x["options"], selected["auto_convert"]) \
+                    if "options" in x.keys() else []
+                self._discord.dispatch("slash_command", ctx)
+                try:
+                    await selected["func"](ctx, *args)
+                except Exception as ex:
+                    await self.on_slash_command_error(ctx, ex)
+                return
+        selected = base[sub_name]
+        args = await self.process_options(ctx.guild, sub_opts, selected["auto_convert"]) \
+            if "options" in sub.keys() else []
+        self._discord.dispatch("slash_command", ctx)
+        try:
+            await selected["func"](ctx, *args)
+        except Exception as ex:
+            await self.on_slash_command_error(ctx, ex)
+
+    async def on_slash_command_error(self, ctx, ex):
+        """
+        Handles Exception occurred from invoking command.
+
+        Example of adding event:
+
+        .. code-block:: python
+
+            @client.event
+            async def on_slash_command_error(ctx, ex):
+                ...
+
+        Example of adding listener:
+
+        .. code-block:: python
+
+            @bot.listen()
+            async def on_slash_command_error(ctx, ex):
+                ...
+
+        :param ctx: Context of the command.
+        :type ctx: :class:`.model.SlashContext`
+        :param ex: Exception from the command invoke.
+        :type ex: Exception
+        :return:
+        """
+        if self.has_listener:
+            if self._discord.extra_events.get('on_slash_command_error'):
+                self._discord.dispatch("slash_command_error", ctx, ex)
+                return
+        if hasattr(self._discord, "on_slash_command_error"):
+            self._discord.dispatch("slash_command_error", ctx, ex)
+            return
+        # Prints exception if not overrided or has no listener for error.
+        self.logger.exception(f"An exception has occurred while executing command `{ctx.name}`:")
