@@ -14,43 +14,64 @@ class SlashContext:
         Do not manually init this model.
 
     :ivar name: Name of the command.
+    :ivar subcommand_name: Subcommand of the command.
+    :ivar subcommand_group: Subcommand group of the command.
     :ivar interaction_id: Interaction ID of the command message.
     :ivar command_id: ID of the command.
     :ivar _http: :class:`.http.SlashCommandRequest` of the client.
     :ivar _discord: :class:`discord.ext.commands.Bot`
+    :ivar logger: Logger instance.
     :ivar sent: Whether you sent the initial response.
-    :ivar guild: :class:`discord.Guild` instance of the command message.
-    :ivar author: :class:`discord.Member` instance representing author of the command message.
-    :ivar channel: :class:`discord.TextChannel` instance representing channel of the command message.
+    :ivar guild: :class:`discord.Guild` instance or guild ID of the command message.
+    :ivar author: :class:`discord.Member` instance or user ID representing author of the command message.
+    :ivar channel: :class:`discord.TextChannel` instance or channel ID representing channel of the command message.
     """
 
     def __init__(self,
                  _http: http.SlashCommandRequest,
                  _json: dict,
-                 _discord: typing.Union[discord.Client, commands.Bot]):
+                 _discord: typing.Union[discord.Client, commands.Bot],
+                 logger):
         self.__token = _json["token"]
         self.name = _json["data"]["name"]
+        self.subcommand_name = None
+        self.subcommand_group = None
         self.interaction_id = _json["id"]
         self.command_id = _json["data"]["id"]
         self._http = _http
         self._discord = _discord
+        self.logger = logger
         self.sent = False
-        self.guild: discord.Guild = _discord.get_guild(int(_json["guild_id"]))
-        self.author: discord.Member = self.guild.get_member(int(_json["member"]["user"]["id"])) if self.guild else None
-        self.channel = self.guild.get_channel(int(_json["channel_id"])) if self.guild else None
+        self.guild: typing.Union[discord.Guild, int] = _discord.get_guild(int(_json["guild_id"]))
+        self.author: typing.Union[discord.Member, int] = self.guild.get_member(int(_json["member"]["user"]["id"])) \
+            if self.guild else None
+        self.channel: typing.Union[discord.TextChannel, int] = self.guild.get_channel(int(_json["channel_id"])) \
+            if self.guild else None
+        if not self.author:
+            self.author = int(_json["member"]["user"]["id"])
+        if not self.channel:
+            self.channel = int(_json["channel_id"])
+        if not self.guild:
+            # Should be set after every others are set.
+            self.guild = int(_json["guild_id"])
 
     async def send(self,
                    send_type: int = 4,
                    content: str = "",
+                   *,
                    embeds: typing.List[discord.Embed] = None,
                    tts: bool = False,
                    allowed_mentions: discord.AllowedMentions = None,
-                   hidden: bool = False):
+                   hidden: bool = False,
+                   complete_hidden: bool = False):
         """
         Sends response of the slash command.
 
         .. note::
-            Param ``hidden`` ONLY works without embeds.
+            Every args except `send_type` and `content` must be passed as keyword args.
+
+        .. warning::
+            Param ``hidden`` only works without embeds.
 
         :param send_type: Type of the response. Refer Discord API DOCS for more info about types. Default ``4``.
         :type send_type: int
@@ -63,10 +84,17 @@ class SlashContext:
         :param allowed_mentions: AllowedMentions of the message.
         :type allowed_mentions: discord.AllowedMentions
         :param hidden: Whether the message is hidden, which means message content will only be seen to the author.
+        :type hidden: bool
+        :param complete_hidden: If this is ``True``, it will be both hidden and `send_type` will be 3. Default ``False``.
+        :type complete_hidden: bool
         :return: ``None``
         """
         if embeds and len(embeds) > 10:
             raise error.IncorrectFormat("Embed must be 10 or fewer.")
+        if complete_hidden:
+            # Overrides both `hidden` and `send_type`.
+            hidden = True
+            send_type = 3
         base = {
             "type": send_type,
             "data": {
@@ -88,12 +116,17 @@ class SlashContext:
                 base["flags"] = 64
             else:
                 base["data"]["flags"] = 64
+        if hidden and embeds:
+            self.logger.warning("You cannot use both `hidden` and `embeds` at the same time!")
+        if (send_type == 2 or send_type == 5) and not self.sent:
+            base = {"type": send_type}
         initial = True if not self.sent else False
         resp = await self._http.post(base, self._discord.user.id, self.interaction_id, self.__token, initial)
         self.sent = True
         return resp
 
     async def edit(self,
+                   *,
                    message_id: typing.Union[int, str] = "@original",
                    content: str = "",
                    embeds: typing.List[discord.Embed] = None,
@@ -102,6 +135,9 @@ class SlashContext:
         """
         Edits response of the slash command.
 
+        .. note::
+            All args must be passed as keyword args.
+
         :param message_id: Response message ID. Default initial message.
         :param content: Text of the response. Can be ``None``.
         :type content: str
